@@ -1,4 +1,34 @@
-# Copyright (c) 2009, Karoly Lorentey  <karoly@lorentey.hu>
+#
+# id3.py
+# From the stagger project: http://code.google.com/p/stagger/
+#
+# Copyright (c) 2009-2011 Karoly Lorentey  <karoly@lorentey.hu>
+# All rights reserved.
+# 
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+# 
+# - Redistributions of source code must retain the above copyright
+#   notice, this list of conditions and the following disclaimer.
+# 
+# - Redistributions in binary form must reproduce the above copyright
+#   notice, this list of conditions and the following disclaimer in
+#   the documentation and/or other materials provided with the
+#   distribution.
+# 
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
 
 """List of frames defined in the various ID3 versions.
 """
@@ -928,6 +958,32 @@ class TSOC(TextFrame):
 @frameclass
 class TSC(TSOC): pass
 
+@frameclass
+class NCON(Frame):
+    """MusicMatch data"""
+    _framespec = (BinaryDataSpec("data"),)
+    _nonstandard = True
+
+@frameclass
+class RGAD(Frame):
+    """Replay Gain Adjustment"""
+    # http://replaygain.hydrogenaudio.org/file_format_id3v2.html
+    _framespec = (IntegerSpec("peak", 32), 
+                  IntegerSpec("radio", 16), 
+                  IntegerSpec("audiophile", 16))
+    _nonstandard = True
+
+@frameclass
+class TTSE(TextFrame):
+    """Lame encoder command line"""
+    _nonstandard = True
+
+@frameclass
+class XSOP(TextFrame):
+    """MusicBrainz Artist Sortname"""
+    _nonstandard = True
+    _version = 3
+
 # ID3v1 genre list
 genres = (
     "Blues", "Classic Rock", "Country", "Dance", "Disco", "Funk", "Grunge",
